@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -98,12 +98,12 @@ rl_gets (char *prompt)
   /* allocate buffer if not yet */
   if (! line_read) {
     if ((line_read = malloc(MAXLINELEN)) == NULL) {
-      j_printerr("memory exceeded\n");
+      fprintf(stderr, "memory exceeded\n");
       exit(1);
     }
   }
   /* Get a line from the user. */
-  j_printerr("%s", prompt);
+  fprintf(stderr, "%s", prompt);
   if (fgets(line_read, MAXLINELEN, stdin) == NULL) { /* input error */
     return NULL;
   }
