@@ -12,7 +12,7 @@
  * @author Akinobu LEE
  * @date   Mon Mar  7 23:19:14 2005
  *
- * $Revision: 1.12 $
+ * $Revision: 1.16 $
  * 
  */
 /*
@@ -24,6 +24,10 @@
 
 /* should be included after all include files */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* backtrellis.c */
 void bt_init(BACKTRELLIS *bt);
 void bt_prepare(BACKTRELLIS *bt);
@@ -102,6 +106,8 @@ boolean dfa_look_around(NEXTWORD *nword, NODE *hypo, RecogProcess *r);
 void segment_set_last_nword(NODE *hypo, RecogProcess *r);
 void pass2_finalize_on_no_result(RecogProcess *r, boolean use_1pass_as_final);
 void wchmm_fbs(HTK_Param *param, RecogProcess *r, int cate_bgn, int cate_num);
+void wchmm_fbs_prepare(RecogProcess *r);
+void wchmm_fbs_free(RecogProcess *r);
 
 /* search_bestfirst_v?.c */
 void clear_stocker(StackDecode *s);
@@ -271,7 +277,7 @@ boolean adin_thread_create(Recog *recog);
 boolean adin_thread_cancel(Recog *recog);
 int adin_go(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Recog *recog);
 boolean adin_standby(ADIn *a, int freq, void *arg);
-boolean adin_begin(ADIn *a);
+boolean adin_begin(ADIn *a, char *file_or_dev_name);
 boolean adin_end(ADIn *a);
 void adin_free_param(Recog *recog);
 
@@ -318,3 +324,8 @@ boolean mfc_module_standby(MFCCCalc *mfcc);
 boolean mfc_module_begin(MFCCCalc *mfcc);
 boolean mfc_module_end(MFCCCalc *mfcc);
 int mfc_module_read(MFCCCalc *mfcc, int *new_t);
+char *mfc_module_input_name(MFCCCalc *mfcc);
+
+#ifdef __cplusplus
+}
+#endif
