@@ -12,7 +12,7 @@
  * @author Akinobu Lee
  * @date   Wed Aug  8 14:53:53 2007
  *
- * $Revision: 1.11 $
+ * $Revision: 1.14 $
  * 
  */
 
@@ -493,12 +493,13 @@ int
 j_open_stream(Recog *recog, char *file_or_dev_name)
 {
   Jconf *jconf;
+  char *p;
 
   jconf = recog->jconf;
 
   if (jconf->input.type == INPUT_WAVEFORM) {
     /* begin A/D input */
-    if (adin_begin(recog->adin) == FALSE) {
+    if (adin_begin(recog->adin, file_or_dev_name) == FALSE) {
       return -2;
     }
     /* create A/D-in thread here */
@@ -509,11 +510,13 @@ j_open_stream(Recog *recog, char *file_or_dev_name)
       }
     }
 #endif
+    /* when using adin func, input name should be obtained when called */
   } else {
     switch(jconf->input.speech_input) {
     case SP_MFCMODULE:
       param_init_content(recog->mfcclist->param);
       if (mfc_module_begin(recog->mfcclist) == FALSE) return -2;
+      /* when using mfc module func, input name should be obtained when called */
       break;
     case SP_MFCFILE:
       /* read parameter file */
@@ -528,12 +531,24 @@ j_open_stream(Recog *recog, char *file_or_dev_name)
       }
       /* output frame length */
       callback_exec(CALLBACK_STATUS_PARAM, recog);
+      /* store the input filename here */
+      strncpy(recog->adin->current_input_name, file_or_dev_name, MAXPATHLEN);
       break;
     default:
       jlog("ERROR: j_open_stream: none of SP_MFC_*??\n");
       return -1;
     }
   }
+
+  if (jconf->input.speech_input != SP_MFCFILE) {
+    /* store current input name using input source specific function */
+    p = j_get_current_filename(recog);
+    if (p) {
+      strncpy(recog->adin->current_input_name, p, MAXPATHLEN);
+    } else {
+      recog->adin->current_input_name[0] = '\0';
+    }
+  }
       
   return 0;
 
@@ -566,11 +581,19 @@ j_close_stream(Recog *recog)
   if (jconf->input.type == INPUT_WAVEFORM) {
 #ifdef HAVE_PTHREAD
     /* close A/D-in thread here */
-    if (recog->adin->enable_thread && ! recog->adin->input_side_segment) {
-      if (adin_thread_cancel(recog) == FALSE) {
-	return -2;
+    if (! recog->adin->input_side_segment) {
+      if (recog->adin->enable_thread) {
+	if (adin_thread_cancel(recog) == FALSE) {
+	  return -2;
+	}
+      } else {
+	recog->adin->end_of_stream = TRUE;
       }
     }
+#else
+    if (! recog->adin->input_side_segment) {
+      recog->adin->end_of_stream = TRUE;
+    }
 #endif
     /* end A/D input */
     if (adin_end(recog->adin) == FALSE) {
