@@ -31,7 +31,7 @@
  * @author Akinobu Lee
  * @date   Mon Sep 19 23:39:15 2005
  *
- * $Revision: 1.5 $
+ * $Revision: 1.7 $
  * 
  */
 /*
@@ -1652,7 +1652,7 @@ build_wchmm(WCHMM_INFO *wchmm, JCONF_LM *lmconf)
     /* ǤĹǤͭñõ */
     maxsharelen=0;
     for (j=0;j<i;j++) {
-      if (wchmm->category_tree && wchmm->lmvar == LM_DFA_GRAMMAR) {
+      if (wchmm->category_tree  && wchmm->lmtype == LM_DFA) {
 	if (wchmm->winfo->wton[i] != wchmm->winfo->wton[j]) continue;
       }
       sharelen = wchmm_check_match(wchmm->winfo, i, j);
@@ -1699,15 +1699,17 @@ build_wchmm(WCHMM_INFO *wchmm, JCONF_LM *lmconf)
 
   /* factoringѤ˳ƾ֤˸³ñΥꥹȤղä */
   if (!wchmm->category_tree) {
-    make_successor_list(wchmm);
 
 #ifdef UNIGRAM_FACTORING
     if (wchmm->lmtype == LM_PROB) {
-      /* äfactoringͤ׻ */
-      /* üʳscɬפʤΤǥե꡼ */
-      calc_all_unigram_factoring_values(wchmm);
+      /* Ʊäfactoringͤ׻ */
+      make_successor_list_unigram_factoring(wchmm);
       jlog("STAT:  1-gram factoring values has been pre-computed\n");
+    } else {
+      make_successor_list(wchmm);
     }
+#else 
+    make_successor_list(wchmm);
 #endif /* UNIGRAM_FACTORING */
     
     if (wchmm->hmminfo->multipath) {
@@ -1722,6 +1724,12 @@ build_wchmm(WCHMM_INFO *wchmm, JCONF_LM *lmconf)
     }
 #endif /* UNIGRAM_FACTORING */
 
+    /* sclist2node is no longer used */
+    if (wchmm->sclist2node != NULL) {
+      free(wchmm->sclist2node);
+      wchmm->sclist2node = NULL;
+    }
+
   }
 
   jlog("STAT: done\n");
@@ -1811,7 +1819,7 @@ build_wchmm2(WCHMM_INFO *wchmm, JCONF_LM *lmconf)
   windex = (WORD_ID *)mymalloc(sizeof(WORD_ID) * wchmm->winfo->num);
   for(i=0;i<wchmm->winfo->num;i++) windex[i] = i;
 
-  if (wchmm->category_tree && wchmm->lmvar == LM_DFA_GRAMMAR) {
+  if (wchmm->category_tree && wchmm->lmtype == LM_DFA) {
 
     /* sort by category -> sort by word ID in each category */
     wchmm_sort_idx_by_category(wchmm->winfo, windex, wchmm->winfo->num);
@@ -1903,7 +1911,7 @@ build_wchmm2(WCHMM_INFO *wchmm, JCONF_LM *lmconf)
       ret = wchmm_add_word(wchmm, i, 0, 0, lmconf->enable_iwsp);
     } else {
       /* the previous word (last_i) is always the most matched one */
-      if (wchmm->category_tree && wchmm->lmvar == LM_DFA_GRAMMAR) {
+      if (wchmm->category_tree && wchmm->lmtype == LM_DFA) {
 	if (wchmm->winfo->wton[i] != wchmm->winfo->wton[last_i]) {
 	  ret = wchmm_add_word(wchmm, i, 0, 0, lmconf->enable_iwsp);
 	} else {
@@ -1945,14 +1953,17 @@ build_wchmm2(WCHMM_INFO *wchmm, JCONF_LM *lmconf)
 
   /* make successor list for all branch nodes for N-gram factoring */
   if (!wchmm->category_tree) {
-    make_successor_list(wchmm);
 
 #ifdef UNIGRAM_FACTORING
     if (wchmm->lmtype == LM_PROB) {
       /* for 1-gram factoring, we can compute the values before search */
-      calc_all_unigram_factoring_values(wchmm);
-      jlog("STAT:  1-gram factoring values has been pre-computed\n");
+       make_successor_list_unigram_factoring(wchmm);
+       jlog("STAT:  1-gram factoring values has been pre-computed\n");
+    } else {
+      make_successor_list(wchmm);
     }
+#else
+    make_successor_list(wchmm);
 #endif /* UNIGRAM_FACTORING */
     if (wchmm->hmminfo->multipath) {
       /* Copy the factoring data according to the skip transitions and startword nodes */
@@ -1965,6 +1976,12 @@ build_wchmm2(WCHMM_INFO *wchmm, JCONF_LM *lmconf)
     }
 #endif /* UNIGRAM_FACTORING */
 
+    /* sclist2node is no longer used */
+    if (wchmm->sclist2node != NULL) {
+      free(wchmm->sclist2node);
+      wchmm->sclist2node = NULL;
+    }
+
   }
 
   //jlog("STAT: done\n");
