@@ -1,7 +1,5 @@
 /**
  * @file   machines.h
- * @author Akinobu LEE
- * @date   Fri Feb 11 03:38:31 2005
  *
  * <EN>
  * @brief  Machine-dependent definitions
@@ -20,13 +18,16 @@
  *
  * @sa config.h.in
  *
- * $Revision: 1.2 $
+ * @author Akinobu LEE
+ * @date   Fri Feb 11 03:38:31 2005
+ *
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -36,8 +37,14 @@
 #include <sent/stddefs.h>
 
 #ifndef HAVE_STRCASECMP
+#ifdef __cplusplus
+extern "C" {
+#endif
 int strcasecmp(char *s1, char *s2);
 int strncasecmp(char *s1, char *s2, size_t n);
+#ifdef __cplusplus
+}
+#endif
 #endif
 
 #endif /* __SENT_MACHINES_H__ */
