@@ -1,7 +1,5 @@
 /**
  * @file   addlog.c
- * @author Akinobu LEE
- * @date   Thu Feb 17 13:23:50 2005
  * 
  * <JA>
  * @brief  пͤι®»ؿ
@@ -11,13 +9,16 @@
  * @brief  Rapid addition of log values
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 13:23:50 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -44,13 +45,13 @@ make_log_tbl()
   int i;
 
   if (built_tbl == FALSE) {
-    j_printerr("Generating addlog table...");
+    jlog("Stat: addlog: generating addlog table (size = %d kB)\n", (TBLSIZE * sizeof(LOGPROB)) / 1024);
     for (i=0;i<TBLSIZE;i++){
       f = - ((float)VRANGE * (float)i / (float)TBLSIZE);
       tbl[i] = log(1 + exp(f));
       /*if (i < 10 || i > TBLSIZE - 10) j_printf("%f: %d(%f)\n", f, i, tbl[i]);*/
     }
-    j_printerr("%d kb...done\n",(TBLSIZE * sizeof(LOGPROB)) / 1024);
+    jlog("Stat: addlog: addlog table generated\n");
     built_tbl = TRUE;
   }
 }
@@ -77,14 +78,14 @@ addlog(LOGPROB x, LOGPROB y)
     if ((tmp = x - y) < LOG_ADDMIN) return y;
     else {
       idx = (unsigned int)((- tmp) * TMAG + 0.5);
-      /* j_printf("%f == %f\n",tbl[idx],log(1 + exp(tmp))); */
+      /* jlog("%f == %f\n",tbl[idx],log(1 + exp(tmp))); */
       return (y + tbl[idx]);
     }
   } else {
     if ((tmp = y - x) < LOG_ADDMIN) return x;
     else {
       idx =(unsigned int)((- tmp) * TMAG + 0.5);
-      /* j_printf("%f == %f\n",tbl[idx],log(1 + exp(tmp))); */
+      /* jlog("%f == %f\n",tbl[idx],log(1 + exp(tmp))); */
       return (x + tbl[idx]);
     }
   }
