@@ -1,7 +1,5 @@
 /**
  * @file   gms.c
- * @author Akinobu LEE
- * @date   Thu Feb 17 14:52:18 2005
  * 
  * <JA>
  * @brief  Gaussian Mixture Selection ˤٷ׻
@@ -15,13 +13,16 @@
  * See the comments in the source for details about implementation.
  * </EN>
  * 
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 14:52:18 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -68,110 +69,97 @@
 #include <sent/htk_hmm.h>
 #include <sent/htk_param.h>
 #include <sent/hmm.h>
-#include <sent/gprune.h>
-#include "globalvars.h"
+#include <sent/hmm_calc.h>
 
 #undef NORMALIZE_GS_SCORE	/* normalize score (ad-hoc) */
 
   /* GS HMMs must be defined at STATE level using "~s NAME" macro,
      where NAMES are like "i:4m", "s2m", etc. */
 
-/* variables for GMS */
-static int my_nbest;		///< Number of states to be selected
-static int allocframenum;	///< Allocated number of frame for storing fallback scores per frame
-
-/* GMS info */
-static GS_SET *gsset;		///< Set of GS states
-static int gsset_num;		///< Num of above
-static int *state2gs; ///< Mapping from triphone state id to gs id
-
-/* results */
-static boolean *is_selected;	///< TRUE if the frame is already selected
-static LOGPROB **fallback_score = NULL; ///< [t][gssetid], LOG_ZERO if selected
-
-/* for calculation */
-static int *gsindex;		///< Index buffer
-static LOGPROB *t_fs;		///< Current fallback_score
-
 
 /** 
  * Register all state defs in GS HMM to GS_SET.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 static void
-build_gsset()
+build_gsset(HMMWork *wrk)
 {
   HTK_HMM_State *st;
 
   /* allocate */
-  gsset = (GS_SET *)mymalloc(sizeof(GS_SET) * OP_gshmm->totalstatenum);
-  gsset_num = OP_gshmm->totalstatenum;
+  wrk->gsset = (GS_SET *)mymalloc(sizeof(GS_SET) * wrk->OP_gshmm->totalstatenum);
+  wrk->gsset_num = wrk->OP_gshmm->totalstatenum;
   /* make ID */
-  for(st = OP_gshmm->ststart; st; st=st->next) {
-    gsset[st->id].state = st;
+  for(st = wrk->OP_gshmm->ststart; st; st=st->next) {
+    wrk->gsset[st->id].state = st;
   }
 }
 
 /**
  * Free gsset.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 static void
-free_gsset()
+free_gsset(HMMWork *wrk)
 {
-  free(gsset);
+  free(wrk->gsset);
 }
 
-#define MAXHMMNAMELEN 40	///< Assumed maximum length of %HMM name
-
 /** 
  * Build the correspondence from GS states to triphone states.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  * @return TRUE on success, FALSE on failure.
  */
 static boolean
-build_state2gs()
+build_state2gs(HMMWork *wrk)
 {
   HTK_HMM_Data *dt;
   HTK_HMM_State *st, *cr;
   int i;
-  char gstr[MAXHMMNAMELEN], cbuf[MAXHMMNAMELEN];
+  char gstr[MAX_HMMNAME_LEN], cbuf[MAX_HMMNAME_LEN];
   boolean ok_p = TRUE;
 
   /* initialize */
-  state2gs = (int *)mymalloc(sizeof(int) * OP_hmminfo->totalstatenum);
-  for(i=0;i<OP_hmminfo->totalstatenum;i++) state2gs[i] = -1;
+  wrk->state2gs = (int *)mymalloc(sizeof(int) * wrk->OP_hmminfo->totalstatenum);
+  for(i=0;i<wrk->OP_hmminfo->totalstatenum;i++) wrk->state2gs[i] = -1;
 
   /* parse through all HMM macro to register their state */
-  for(dt = OP_hmminfo->start; dt; dt=dt->next) {
-    if (strlen(dt->name) >= MAXHMMNAMELEN - 2) {
-      j_printerr("Error: too long hmm name (>%d): \"%s\"\n",
-		 MAXHMMNAMELEN-3, dt->name);
+  for(dt = wrk->OP_hmminfo->start; dt; dt=dt->next) {
+    if (strlen(dt->name) >= MAX_HMMNAME_LEN - 2) {
+      jlog("Error: gms: too long hmm name (>%d): \"%s\"\n",
+	   MAX_HMMNAME_LEN-3, dt->name);
+      jlog("Error: gms: change value of MAX_HMMNAME_LEN\n");
       ok_p = FALSE;
       continue;
     }
     for(i=1;i<dt->state_num-1;i++) { /* for all state */
       st = dt->s[i];
       /* skip if already assigned */
-      if (state2gs[st->id] != -1) continue;
+      if (wrk->state2gs[st->id] != -1) continue;
       /* set corresponding gshmm name */
       sprintf(gstr, "%s%dm", center_name(dt->name, cbuf), i + 1);
       /* look up the state in OP_gshmm */
-      if ((cr = state_lookup(OP_gshmm, gstr)) == NULL) {
-	j_printerr("Error: GS HMM \"%s\" not defined\n", gstr);
+      if ((cr = state_lookup(wrk->OP_gshmm, gstr)) == NULL) {
+	jlog("Error: gms: GS HMM \"%s\" not defined\n", gstr);
 	ok_p = FALSE;
 	continue;
       }
       /* store its ID */
-      state2gs[st->id] = cr->id;
+      wrk->state2gs[st->id] = cr->id;
     }
   }
 #ifdef PARANOIA
   {
     HTK_HMM_State *st;
-    for(st=OP_hmminfo->ststart; st; st=st->next) {
+    for(st=wrk->OP_hmminfo->ststart; st; st=st->next) {
       printf("%s -> %s\n", (st->name == NULL) ? "(NULL)" : st->name,
-	     (gsset[state2gs[st->id]].state)->name);
+	     (wrk->gsset[wrk->state2gs[st->id]].state)->name);
     }
   }
 #endif
@@ -181,31 +169,42 @@ build_state2gs()
 /**
  * free state2gs.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 static void
-free_state2gs()
+free_state2gs(HMMWork *wrk)
 {
-  free(state2gs);
+  free(wrk->state2gs);
 }
 
 
 /* sort to find N-best states */
-#define SD(A) gsindex[A-1]	///< Index macro for heap sort
+#define SD(A) idx[A-1]	///< Index macro for heap sort
 #define SCOPY(D,S) D = S	///< Element copy macro for heap sort
-#define SVAL(A) (t_fs[gsindex[A-1]]) ///< Element evaluation macro for heap sort
-#define STVAL (t_fs[s]) ///< Element current value macro for heap sort
+#define SVAL(A) (fs[idx[A-1]]) ///< Element evaluation macro for heap sort
+#define STVAL (fs[s]) ///< Element current value macro for heap sort
 
 /** 
  * Heap sort of @a gsindex to determine which model gets N best likelihoods.
  * 
- * @param neednum [in] needed number N
- * @param totalnum [in] total number of data
+ * @param wrk [i/o] HMM computation work area
+ *
  */
 static void
-sort_gsindex_upward(int neednum, int totalnum)
+sort_gsindex_upward(HMMWork *wrk)
 {
   int n,root,child,parent;
   int s;
+  int *idx;
+  LOGPROB *fs;
+  int neednum, totalnum;
+
+  idx = wrk->gsindex;
+  fs = wrk->t_fs;
+  neednum = wrk->my_nbest;
+  totalnum = wrk->gsset_num;
+
   for (root = totalnum/2; root >= 1; root--) {
     SCOPY(s, SD(root));
     parent = root;
@@ -244,27 +243,29 @@ sort_gsindex_upward(int neednum, int totalnum)
 /** 
  * Calculate all GS state scores and select the best ones.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 static void
-do_gms()
+do_gms(HMMWork *wrk)
 {
   int i;
   
   /* compute all gshmm scores (in gs_score.c) */
-  compute_gs_scores(gsset, gsset_num, t_fs);
+  compute_gs_scores(wrk);
   /* sort and select */
-  sort_gsindex_upward(my_nbest, gsset_num);
-  for(i=gsset_num - my_nbest;i<gsset_num;i++) {
+  sort_gsindex_upward(wrk);
+  for(i=wrk->gsset_num - wrk->my_nbest;i<wrk->gsset_num;i++) {
     /* set scores of selected states to LOG_ZERO */
-    t_fs[gsindex[i]] = LOG_ZERO;
+    wrk->t_fs[wrk->gsindex[i]] = LOG_ZERO;
   }
 
   /* power e -> 10 */
 #ifdef NORMALIZE_GS_SCORE
   /* normalize other fallback scores (rate of max) */
-  for(i=0;i<gsset_num;i++) {
-    if (t_fs[i] != LOG_ZERO) {
-      t_fs[i] = t_fs[i] * 0.975;
+  for(i=0;i<wrk->gsset_num;i++) {
+    if (wrk->t_fs[i] != LOG_ZERO) {
+      wrk->t_fs[i] *= 0.975;
     }
   }
 #endif
@@ -274,49 +275,45 @@ do_gms()
 /** 
  * Initialize the GMS related functions and data.
  * 
- * @param nbest [in] N-best state to compute the precise triphone.
+ * @param wrk [i/o] HMM computation work area
  * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-gms_init(int nbest)
+gms_init(HMMWork *wrk)
 {
   int i;
   
   /* Check gshmm type */
-  if (OP_gshmm->is_triphone) {
-    j_printerr("Error: GS HMM should be a monophone model\n");
+  if (wrk->OP_gshmm->is_triphone) {
+    jlog("Error: gms: GS HMM should be a monophone model\n");
     return FALSE;
   }
-  if (OP_gshmm->is_tied_mixture) {
-    j_printerr("Error: GS HMM should not be a tied mixture model\n");
+  if (wrk->OP_gshmm->is_tied_mixture) {
+    jlog("Error: gms: GS HMM should not be a tied mixture model\n");
     return FALSE;
   }
 
-  /* store as local info */
-  my_nbest = nbest;
-
   /* Register all GS HMM states in GS_SET */
-  build_gsset();
+  build_gsset(wrk);
   /* Make correspondence of all triphone states to GS HMM states */
-  j_printerr("Mapping HMM states to GS HMM...");
-  if (build_state2gs() == FALSE) {
-    j_printerr("Error: failed in assigning GS HMM state for each state\n");
+  if (build_state2gs(wrk) == FALSE) {
+    jlog("Error: gms: failed in assigning GS HMM state for each state\n");
     return FALSE;
   }
-  j_printerr("done\n");
+  jlog("Stat: gms: GS HMMs are mapped to HMM states\n");
 
   /* prepare index buffer for heap sort */
-  gsindex = (int *)mymalloc(sizeof(int) * gsset_num);
-  for(i=0;i<gsset_num;i++) gsindex[i] = i;
+  wrk->gsindex = (int *)mymalloc(sizeof(int) * wrk->gsset_num);
+  for(i=0;i<wrk->gsset_num;i++) wrk->gsindex[i] = i;
 
   /* init cache status */
-  fallback_score = NULL;
-  is_selected = NULL;
-  allocframenum = -1;
+  wrk->fallback_score = NULL;
+  wrk->gms_is_selected = NULL;
+  wrk->gms_allocframenum = -1;
 
   /* initialize gms_gprune functions */
-  gms_gprune_init(OP_hmminfo, gsset_num);
+  gms_gprune_init(wrk);
   
   return TRUE;
 }
@@ -324,36 +321,37 @@ gms_init(int nbest)
 /** 
  * Setup GMS parameters for next input.
  * 
+ * @param wrk [i/o] HMM computation work area
  * @param framenum [in] length of next input in frames
  * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-gms_prepare(int framenum)
+gms_prepare(HMMWork *wrk, int framenum)
 {
   LOGPROB *tmp;
   int t;
 
   /* allocate cache */
-  if (allocframenum < framenum) {
-    if (fallback_score != NULL) {
-      free(fallback_score[0]);
-      free(fallback_score);
-      free(is_selected);
+  if (wrk->gms_allocframenum < framenum) {
+    if (wrk->fallback_score != NULL) {
+      free(wrk->fallback_score[0]);
+      free(wrk->fallback_score);
+      free(wrk->gms_is_selected);
     }
-    fallback_score = (LOGPROB **)mymalloc(sizeof(LOGPROB *) * framenum);
-    tmp = (LOGPROB *)mymalloc(sizeof(LOGPROB) * gsset_num * framenum);
+    wrk->fallback_score = (LOGPROB **)mymalloc(sizeof(LOGPROB *) * framenum);
+    tmp = (LOGPROB *)mymalloc(sizeof(LOGPROB) * wrk->gsset_num * framenum);
     for(t=0;t<framenum;t++) {
-      fallback_score[t] = &(tmp[gsset_num * t]);
+      wrk->fallback_score[t] = &(tmp[wrk->gsset_num * t]);
     }
-    is_selected = (boolean *)mymalloc(sizeof(boolean) * framenum);
-    allocframenum = framenum;
+    wrk->gms_is_selected = (boolean *)mymalloc(sizeof(boolean) * framenum);
+    wrk->gms_allocframenum = framenum;
   }
   /* clear */
-  for(t=0;t<framenum;t++) is_selected[t] = FALSE;
+  for(t=0;t<framenum;t++) wrk->gms_is_selected[t] = FALSE;
 
   /* prepare gms_gprune functions */
-  gms_gprune_prepare();
+  gms_gprune_prepare(wrk);
   
   return TRUE;
 }
@@ -361,19 +359,21 @@ gms_prepare(int framenum)
 /**
  * Free GMS related work areas.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  */
 void
-gms_free()
+gms_free(HMMWork *wrk)
 {
-  free_gsset();
-  free_state2gs();
-  free(gsindex);
-  if (fallback_score != NULL) {
-    free(fallback_score[0]);
-    free(fallback_score);
-    free(is_selected);
+  free_gsset(wrk);
+  free_state2gs(wrk);
+  free(wrk->gsindex);
+  if (wrk->fallback_score != NULL) {
+    free(wrk->fallback_score[0]);
+    free(wrk->fallback_score);
+    free(wrk->gms_is_selected);
   }
-  gms_gprune_free();
+  gms_gprune_free(wrk);
 }
 
 
@@ -386,25 +386,27 @@ gms_free()
  * the GMS %HMM will be returned instead as a fallback score.
  * Else, the precise triphone will be computed and returned.
  * 
+ * @param wrk [i/o] HMM computation work area
+ * 
  * @return the state output probability score in log10.
  */
 LOGPROB
-gms_state()
+gms_state(HMMWork *wrk)
 {
   LOGPROB gsprob;
-  if (OP_last_time != OP_time) { /* different frame */
+  if (wrk->OP_last_time != wrk->OP_time) { /* different frame */
     /* set current buffer */
-    t_fs = fallback_score[OP_time];
+    wrk->t_fs = wrk->fallback_score[wrk->OP_time];
     /* select state if not yet */
-    if (!is_selected[OP_time]) {
-      do_gms();
-      is_selected[OP_time] = TRUE;
+    if (!wrk->gms_is_selected[wrk->OP_time]) {
+      do_gms(wrk);
+      wrk->gms_is_selected[wrk->OP_time] = TRUE;
     }
   }
-  if ((gsprob = t_fs[state2gs[OP_state_id]]) != LOG_ZERO) {
+  if ((gsprob = wrk->t_fs[wrk->state2gs[wrk->OP_state_id]]) != LOG_ZERO) {
     /* un-selected: return the fallback value */
     return(gsprob);
   }
   /* selected: calculate the real outprob of the state */
-  return(calc_outprob());
+  return((*(wrk->calc_outprob))(wrk));
 }
