@@ -1,7 +1,5 @@
 /**
  * @file   voca_malloc.c
- * @author Akinobu LEE
- * @date   Fri Feb 18 21:33:29 2005
  * 
  * <JA>
  * @brief  ñ켭¤ΤΥդȲ
@@ -11,13 +9,16 @@
  * @brief  Memory allocation of word dictionary information
  * </EN>
  * 
- * $Revision: 1.4 $
+ * @author Akinobu LEE
+ * @date   Fri Feb 18 21:33:29 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -49,7 +50,6 @@ word_info_new()
 void
 word_info_free(WORD_INFO *winfo)
 {
-  WORD_ID w;
   /* free each word info */
   if (winfo->mroot != NULL) mybfree2(&(winfo->mroot));
   /* free word info */
@@ -101,16 +101,17 @@ winfo_init(WORD_INFO *winfo)
  * 
  * @param winfo [i/o] word dictionary to be expanded.
  */
-void
+boolean
 winfo_expand(WORD_INFO *winfo)
 {
   int n;
 
   n = winfo->maxnum;
   if (n >= MAX_WORD_NUM) {
-    j_error("Error: maximum dict size exceeded limit (%d)\n", MAX_WORD_NUM);
+    jlog("Error: voca_malloc: maximum dict size exceeded limit (%d)\n", MAX_WORD_NUM);
+    return FALSE;
   }
-  n += MAXWSTEP;
+  n *= 2;
   if (n > MAX_WORD_NUM) n = MAX_WORD_NUM;
 
   winfo->wlen = (unsigned char *)myrealloc(winfo->wlen, sizeof(unsigned char)*n);
@@ -123,5 +124,7 @@ winfo_expand(WORD_INFO *winfo)
 #endif
   winfo->is_transparent = (boolean *)myrealloc(winfo->is_transparent, sizeof(boolean)*n);
   winfo->maxnum = n;
+
+  return TRUE;
 }
 
