@@ -7,7 +7,7 @@
 
 /* mkbinhmm --- read in ascii hmmdefs file and write in binary format */
 
-/* $Id: mkbinhmm.c,v 1.2 2007/12/18 08:45:55 sumomo Exp $ */
+/* $Id: mkbinhmm.c,v 1.4 2008/09/30 03:58:18 sumomo Exp $ */
 
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
@@ -21,7 +21,7 @@ static void
 usage(char *s)
 {
   printf("mkbinhmm: convert HMM definition file to binary format for Julius\n");
-  printf("usage: %s [-C HTKConfig] hmmdefs binhmm\n", s);
+  printf("usage: %s [-htkconf HTKConfig] hmmdefs binhmm\n", s);
   printf("\nLibrary configuration: ");
   confout_version(stdout);
   confout_am(stdout);
@@ -69,7 +69,10 @@ main(int argc, char *argv[])
 
   /* read hmmdef file */
   undef_para(&para);
-  init_hmminfo(hmminfo, infile, NULL, &para);
+  if (init_hmminfo(hmminfo, infile, NULL, &para) == FALSE) {
+    fprintf(stderr, "--- terminated\n");
+    return -1;
+  }
 
   if (conffile != NULL) {
     /* if input HMMDEFS already has embedded parameter
